/*** Manipulation.cpp ***/

#include "MixedDomainFilter.h"
#include <cv.h>
#pragma comment(lib, "cv.lib")
#include <cxcore.h>
#pragma comment(lib, "cxcore.lib")
#include <highgui.h>
#pragma comment(lib, "highgui.lib")
#include <iostream>

void manipulationRGB(string xFileName, string yFileName, double alpha, double beta, double sigma, double lambda, int level) {
	std::cout << ">> Manipulation (RGB): " << yFileName.c_str() << "..." << std::endl;

	IplImage *xImage = cvLoadImage(xFileName.c_str(), 1);
	IplImage *yImage = cvCloneImage(xImage);
	int W = xImage->width;
	int H = xImage->height;
	int C = xImage->nChannels;

	double *x = new double[W*H];
	double *y = new double[W*H];
	computeTable(alpha, beta, sigma, 1.00);
	for(int k = 0;  k < C;  k++) {
		for(int i = 0;  i < W;  i++) {
			for(int j = 0;  j < H;  j++) {
				x[j*W+i] = unsigned char(xImage->imageData[j*xImage->widthStep+C*i+k])/255.00;
			}
		}

		pyramidFilter(W, H, x, y, lambda, alpha, beta, sigma, true, true, level, 1.00);

		for(int i = 0;  i < W;  i++) {
			for(int j = 0;  j < H;  j++) {
				yImage->imageData[j*yImage->widthStep+C*i+k] = char(int(255*y[j*W+i]+0.50));
			}
		}
	}
	cvSaveImage(yFileName.c_str(), yImage);

	cvReleaseImage(&xImage);
	cvReleaseImage(&yImage);
	delete []x;
	delete []y;
}

void manipulationLab(string xFileName, string yFileName, double alpha, double beta, double sigma, double lambda, int level) {
	std::cout << ">> Manipulation (CIELAB): " << yFileName.c_str() << "..." << std::endl;

	IplImage *xImage = cvLoadImage(xFileName.c_str(), 1);
	int W = xImage->width;
	int H = xImage->height;

	IplImage *yImageFloat = cvCreateImage(cvSize(W, H), 32, 3);
	cvCvtScale(xImage, yImageFloat, 1.0/255);
	IplImage *xImageLab = cvCloneImage(xImage);
	IplImage *yImageLabFloat = cvCloneImage(yImageFloat);
	cvCvtColor(yImageFloat, yImageLabFloat, CV_BGR2Lab);

	double *x = new double[W*H];
	double *y = new double[W*H];
	for(int i = 0;  i < W;  i++) {
		for(int j = 0;  j < H;  j++) {
			x[j*W+i] = *((float *)(&yImageLabFloat->imageData[j*yImageLabFloat->widthStep+12*i]))/100;
		}
	}

	computeTable(alpha, beta, sigma, 1.00);
	pyramidFilter(W, H, x, y, lambda, alpha, beta, sigma, true, true, level, 1.00);

	for(int i = 0;  i < W;  i++) {
		for(int j = 0;  j < H;  j++) {
			*((float *)(&yImageLabFloat->imageData[j*yImageLabFloat->widthStep+12*i])) = float(100*y[j*W+i]);
		}
	}
	cvCvtColor(yImageLabFloat, yImageFloat, CV_Lab2BGR);
	cvCvtScale(yImageFloat, xImage, 255.0);
	cvSaveImage(yFileName.c_str(), xImage);

	cvReleaseImage(&xImage);
	cvReleaseImage(&xImageLab);
	cvReleaseImage(&yImageFloat);
	cvReleaseImage(&yImageLabFloat);
	delete []x;
	delete []y;
}
