/*** MixedDomainFilter.h */

#pragma once
#ifndef _MIXED_DOMAIN_FILTER_H
#define _MIXED_DOMAIN_FILTER_H

#include <string>
#include <vector>
using std::string;
using std::vector;

#ifndef M_PI
#define M_PI 3.141592653589793238
#endif

const int N = 4;
const int NK = 4;
const double coeffK[NK+1] = {70.00/256, 56.00/256, 28.00/256, 8.00/256, 1.00/256};
const int MIN_SIZE = 8;

/** MixedDomainFilter */
void linearFilter(int W, int H, double *x, double *y, double beta);
void computeTable(double alpha, double beta, double sigma, double range);
void MixedDomainFilter(int W, int H, double *x, double *y, double *b, double lambda, double alpha, double beta, double sigma, bool useTable, double range);
void localFilter(int W, int H, double *x, int i, int j, double *s, double *t, double alpha, double beta, double sigma, bool useTable, double range);
void jointMixedDomainFilter(int W, int H, double *x, double **ref, double *y, double *b, double lambda, double alpha, double beta, double sigma,
					bool useTable, double range);
void jointLocalFilter(int W, int H, double *x, double **ref, int i, int j, double *s, double *t, double alpha, double beta, double sigma,
					  bool useTable, double range);

/** PyramidFilter */
void pyramidFilter(int W, int H, double *x, double *y, double lambda, double alpha, double beta, double sigma,
				   bool clamp, bool useTable, int level, double range);
void jointPyramidFilter(int W, int H, double *x, double **ref, double *y, double lambda, double alpha, double beta, double sigma,
						bool clamp, bool useTable, int level, double range);

/** Manipulation */
void manipulationRGB(string xFileName, string yFileName, double alpha, double beta, double sigma, double lambda = 1.0, int level = -1);
void manipulationLab(string xFileName, string yFileName, double alpha, double beta, double sigma, double lambda = 1.0, int level = -1);

/** HDRCompression */
void HDRCompression(string xFileName, string yFileName, double alpha, double beta, double sigma, double lambda = 1.0, int level = -1, double percentile = 0.5);
void toneAdjustment(string xFileName, string yFileName, double alpha, double beta, double sigma, double lambda = 1.0, int level = -1, double percentile = 0.5);

/** NPR */
void NPRProcess(string xFileName, string yFileName, double sigma, double lambda = 1.0, int level = -1);

/** DeHaze (by Yan Gu) */
void deHaze(string xFileName, string yFileName, double Rat = 1.0, double RatE = 1.0, double RatL = 0.9, int level = -1);

#endif
