/* NPR.cpp */

#include "MixedDomainFilter.h"
#include <cv.h>
#pragma comment(lib, "cv.lib")
#include <cxcore.h>
#pragma comment(lib, "cxcore.lib")
#include <highgui.h>
#pragma comment(lib, "highgui.lib")
#include <iostream>
using namespace std;

extern double fastSel(double *X, int p, int q, int tar);
void findThreshold(int W, int H, double *x, double &xThreshold, double c) {
	double *a = new double[W*H];
	memcpy(a, x, W*H*sizeof(double));
	xThreshold = fastSel(a, 0, W*H-1, int(W*H*c-0.5));
	delete []a;
}
void dogEdge(IplImage *image, double *yEdge) {
	IplImage *smooth1 = cvCloneImage(image);
	IplImage *smooth2 = cvCloneImage(image);
	cvSmooth(image, smooth1, CV_GAUSSIAN, 7, 7, 1.25);
	cvSmooth(image, smooth2, CV_GAUSSIAN, 7, 7, 2.00);

	int W = image->width;
	int H = image->height;
	int C = image->nChannels;

	double edgeMax = 0.00;
	double *dx = new double[C];
	double *dy = new double[C];
	for(int i = 0;  i < W;  i++) {
		for(int j = 0;  j < H;  j++) {
			yEdge[j*W+i] = 0.00;
			for(int k = 0;  k < C;  k++) {
				dx[k] = int(unsigned char(smooth1->imageData[j*smooth1->widthStep+C*min(i+1, W-1)+k]))-
					int(unsigned char(smooth1->imageData[j*smooth1->widthStep+C*max(i-1, 0)+k]));
				dy[k] = int(unsigned char(smooth1->imageData[min(j+1, H-1)*smooth1->widthStep+C*i+k]))-
					int(unsigned char(smooth1->imageData[max(j-1, 0)*smooth1->widthStep+C*i+k]));
				yEdge[j*W+i] += dx[k]*dx[k]+dy[k]*dy[k];
			}
			yEdge[j*W+i] = sqrt(yEdge[j*W+i]);
			edgeMax = max(edgeMax, yEdge[j*W+i]);
		}
	}
	delete []dx;
	delete []dy;
	for(int index = 0;  index < W*H;  index++) {
		yEdge[index] /= edgeMax;
	}

	double yThreshold;
	findThreshold(W, H, yEdge, yThreshold, 0.80);
	for(int index = 0;  index < W*H;  index++) {
		if(yEdge[index] < yThreshold) {
			yEdge[index] = yThreshold*(tanh(4.00*(yEdge[index]/yThreshold-1.00))+1.00);
		}else {
			yEdge[index] = yThreshold+(1.00-yThreshold)*tanh(4.00*(yEdge[index]-yThreshold)/(1.00-yThreshold));
		}
	}

	cvReleaseImage(&smooth1);
	cvReleaseImage(&smooth2);
}
void NPRProcess(string xFileName, string yFileName, double sigma, double lambda, int level) {
	std::cout << ">> NPR Process (RGB): " << yFileName.c_str() << "..." << std::endl;

	IplImage *xImage = cvLoadImage(xFileName.c_str(), 1);
	IplImage *yImage = cvCloneImage(xImage);
	int W = xImage->width;
	int H = xImage->height;
	int C = xImage->nChannels;

	double *x = new double[W*H];
	double *y = new double[W*H];
	double alpha = 0.00, beta = 1.00;
	computeTable(alpha, beta, sigma, 1.00);
	for(int k = 0;  k < C;  k++) {
		for(int i = 0;  i < W;  i++) {
			for(int j = 0;  j < H;  j++) {
				x[j*W+i] = unsigned char(xImage->imageData[j*xImage->widthStep+C*i+k])/255.00;
			}
		}

		pyramidFilter(W, H, x, y, lambda, alpha, beta, sigma, true, true, level, 1.00);

		for(int i = 0;  i < W;  i++) {
			for(int j = 0;  j < H;  j++) {
				yImage->imageData[j*yImage->widthStep+C*i+k] = char(int(255*y[j*W+i]+0.50));
			}
		}
	}

	double *yEdge = new double[W*H];
	dogEdge(yImage, yEdge);
	for (int i = 0;  i < W;  i++) {
		for (int j = 0;  j < H;  j++) {
			for (int k = 0;  k < C;  k++) {
				int yColor = int(unsigned char(yImage->imageData[j*yImage->widthStep+C*i+k]));
				yImage->imageData[j*yImage->widthStep+C*i+k] = char(yColor*(1.00-yEdge[j*W+i]));
			}
		}
	}
	cvSaveImage(yFileName.c_str(), yImage);

	cvReleaseImage(&xImage);
	cvReleaseImage(&yImage);
	delete []x;
	delete []y;
	delete []yEdge;
}
